@@ -174,10 +174,10 @@ explore <- function(Y,
                     seed = 1, ...){
 
   # temporary warning until missing data is fully implemented
-  # if(type != "continuous"){
-  #   warning(paste0("imputation during model fitting is\n",
-  #                  "currently only implemented for 'continuous' data."))
-  # }
+  if(type != "continuous"){
+    warning(paste0("imputation during model fitting is\n",
+                   "currently only implemented for 'continuous' data."))
+  }
 
   old <- .Random.seed
 
